@@ -1,4 +1,6 @@
 /* graphene-euler.c: Euler angles
+ *
+ * SPDX-License-Identifier: MIT
  *
  * Copyright 2014  Emmanuele Bassi
  *
@@ -100,9 +102,9 @@
 
 #define EULER_DEFAULT_ORDER GRAPHENE_EULER_ORDER_SXYZ
 
-#define LAST_DEPRECATED GRAPHENE_EULER_ORDER_ZYX + 1
+#define LAST_DEPRECATED GRAPHENE_EULER_ORDER_ZYX
 
-#define ORDER_OFFSET(o) ((o) - LAST_DEPRECATED)
+#define ORDER_OFFSET(o) ((o) - (LAST_DEPRECATED + 1))
 
 /* The orders of rotation we support, minus the deprecated aliases */
 enum {
@@ -139,7 +141,8 @@ enum {
   RZYZ
 };
 
-struct axis_param {
+struct axis_param
+{
   /* The initial axis in the permutation */
   int first_axis;
   /* Parity of the axis permutation (false for even, true for odd) */
@@ -234,8 +237,10 @@ euler_to_matrix (float                    ai,
 
   float m[16];
 
-/* Our matrices are row major */
-#define M(m,r,c) (m)[((r) << 2) + (c)]
+/* Our matrices are row major, however the code below is based on code
+   that assumes matrixes apply from the left, and we apply on the
+   right, so need to flip row/column. */
+#define M(m, r, c) (m)[((c) << 2) + (r)]
 
   /* We need to construct the matrix from float values instead
    * of SIMD vectors because the access is parametrised on the
@@ -293,10 +298,13 @@ matrix_to_euler (const graphene_matrix_t *matrix,
    * lane access.
    */
   float m[16];
+
   graphene_matrix_to_float (matrix, m);
 
-/* Our matrices are row major */
-#define M(m,r,c) (m)[((r) << 2) + (c)]
+/* Our matrices are row major, however the code below is based on code
+   that assumes matrixes apply from the left, and we apply on the
+   right, so need to flip row/column. */
+#define M(m, r, c) (m)[((c) << 2) + (r)]
 
   float ax, ay, az;
   if (params->repetition)
@@ -539,6 +547,7 @@ graphene_euler_init_from_matrix (graphene_euler_t        *e,
   if (m == NULL || graphene_matrix_is_identity (m))
     return graphene_euler_init_with_order (e, 0.f, 0.f, 0.f, order);
 
+  order = graphene_euler_get_real_order (order);
   matrix_to_euler (m, &order_parameters[ORDER_OFFSET (order)], &x, &y, &z);
   graphene_euler_init_internal (e, x, y, z, order);
 
@@ -572,7 +581,7 @@ graphene_euler_init_from_quaternion (graphene_euler_t            *e,
 
   graphene_quaternion_to_matrix (q, &m);
 
-  return graphene_euler_init_from_matrix (e, &m, order);
+  return graphene_euler_init_from_matrix (e, &m, graphene_euler_get_real_order (order));
 }
 
 /**
@@ -602,7 +611,7 @@ graphene_euler_init_from_vec3 (graphene_euler_t       *e,
   else
     graphene_vec3_init_from_vec3 (&e->angles, graphene_vec3_zero ());
 
-  e->order = order;
+  e->order = graphene_euler_get_real_order (order);
 
   return e;
 }
@@ -634,6 +643,31 @@ graphene_euler_init_from_euler (graphene_euler_t       *e,
   return e;
 }
 
+/**
+ * graphene_euler_init_from_radians:
+ * @e: the #graphene_euler_t to initialize
+ * @x: rotation angle on the X axis, in radians
+ * @y: rotation angle on the Y axis, in radians
+ * @z: rotation angle on the Z axis, in radians
+ * @order: order of rotations
+ *
+ * Initializes a #graphene_euler_t using the given angles
+ * and order of rotation.
+ *
+ * Returns: (transfer none): the initialized #graphene_euler_t
+ *
+ * Since: 1.10
+ */
+graphene_euler_t *
+graphene_euler_init_from_radians (graphene_euler_t       *e,
+                                  float                   x,
+                                  float                   y,
+                                  float                   z,
+                                  graphene_euler_order_t  order)
+{
+  return graphene_euler_init_internal (e, x, y, z, order);
+}
+
 static bool
 euler_equal (const void *p1,
              const void *p2)
@@ -953,6 +987,10 @@ graphene_euler_to_matrix (const graphene_euler_t *e,
                           graphene_matrix_t      *res)
 {
   graphene_euler_order_t order = graphene_euler_get_real_order (e->order);
+
+  /* We need to use the alpha/beta/gamma accessor to account for
+   * rotations that replicate the first axis on the last
+   */
   float ai = graphene_euler_get_alpha (e);
   float aj = graphene_euler_get_beta (e);
   float ak = graphene_euler_get_gamma (e);
@@ -979,6 +1017,7 @@ graphene_euler_to_quaternion (const graphene_euler_t *e,
 
   float ci, cj, ck;
   float si, sj, sk;
+
   graphene_sincos (ti, &si, &ci);
   graphene_sincos (tj, &sj, &cj);
   graphene_sincos (tk, &sk, &ck);
@@ -1033,5 +1072,5 @@ graphene_euler_reorder (const graphene_euler_t *e,
   graphene_quaternion_t q;
 
   graphene_quaternion_init_from_euler (&q, e);
-  graphene_euler_init_from_quaternion (res, &q, order);
+  graphene_euler_init_from_quaternion (res, &q, graphene_euler_get_real_order (order));
 }
