@@ -47,6 +47,42 @@
  * The contents of a #graphene_matrix_t are private, and direct access is not
  * possible. You can modify and read the contents of a #graphene_matrix_t
  * only through the provided API.
+ *
+ * # Conventions # {#conventions}
+ *
+ * Graphene uses left-multiplication for all its operations on vectors and
+ * matrices; in other words, given a matrix `A` and a vector `b`, the result
+ * of a multiplication is going to be:
+ *
+ * |[
+ *   res = b × A
+ * ]|
+ *
+ * Multiplying two matrices, on the other hand, will use right-multiplication;
+ * given two matrices `A` and `B`, the result of the multiplication is going
+ * to be
+ *
+ * |[
+ *   res = A × B
+ * ]|
+ *
+ * as the implementation will multiply each row vector of matrix `A` with the
+ * matrix `B` to obtain the new row vectors of the result matrix:
+ *
+ * |[
+ *   res = ⎡ A.x × B ⎤
+ *         ⎜ A.y × B ⎟
+ *         ⎜ A.z × B ⎟
+ *         ⎣ A.w × B ⎦
+ * ]|
+ *
+ * For more information, see the documentation for #graphene_simd4x4f_t,
+ * especially the following functions:
+ *
+ *   - graphene_simd4x4f_vec4_mul()
+ *   - graphene_simd4x4f_vec3_mul()
+ *   - graphene_simd4x4f_point3_mul()
+ *   - graphene_simd4x4f_matrix_mul()
  */
 
 #include "graphene-private.h"
@@ -285,6 +321,18 @@ graphene_matrix_init_ortho (graphene_matrix_t *m,
  * coordinates. The top of the camera is aligned to the direction
  * of the @up vector.
  *
+ * Before the transform, the camera is assumed to be placed at the
+ * origin, looking towards the negative Z axis, with the top side of
+ * the camera facing in the direction of the Y axis and the right
+ * side in the direction of the X axis.
+ *
+ * In theory, one could use @m to transform a model of such a camera
+ * into world-space. However, it is more common to use the inverse of
+ * @m to transform another object from world coordinates to the view
+ * coordinates of the camera. Typically you would then apply the
+ * camera projection transform to get from view to screen
+ * coordinates.
+ *
  * Returns: (transfer none): the initialized matrix
  *
  * Since: 1.0
@@ -889,6 +937,8 @@ graphene_matrix_transform_point (const graphene_matrix_t *m,
  * account the fourth row vector of the #graphene_matrix_t when computing
  * the dot product of each row vector of the matrix.
  *
+ * See also: graphene_simd4x4f_point3_mul()
+ *
  * Since: 1.2
  */
 void
@@ -917,6 +967,8 @@ graphene_matrix_transform_point3d (const graphene_matrix_t  *m,
  *
  * The result is a coplanar quadrilateral.
  *
+ * See also: graphene_matrix_transform_point()
+ *
  * Since: 1.0
  */
 void
@@ -925,6 +977,9 @@ graphene_matrix_transform_rect (const graphene_matrix_t *m,
                                 graphene_quad_t         *res)
 {
   graphene_point_t ret[4];
+  graphene_rect_t rr;
+
+  graphene_rect_normalize_r (r, &rr);
 
 #define TRANSFORM_POINT(matrix, rect, corner, out_p)   do {\
   graphene_simd4f_t __s; \
@@ -935,10 +990,10 @@ graphene_matrix_transform_rect (const graphene_matrix_t *m,
   out_p.x = graphene_simd4f_get_x (__s); \
   out_p.y = graphene_simd4f_get_y (__s);           } while (0)
 
-  TRANSFORM_POINT (m, r, top_left, ret[0]);
-  TRANSFORM_POINT (m, r, top_right, ret[1]);
-  TRANSFORM_POINT (m, r, bottom_right, ret[2]);
-  TRANSFORM_POINT (m, r, bottom_left, ret[3]);
+  TRANSFORM_POINT (m, &rr, top_left, ret[0]);
+  TRANSFORM_POINT (m, &rr, top_right, ret[1]);
+  TRANSFORM_POINT (m, &rr, bottom_right, ret[2]);
+  TRANSFORM_POINT (m, &rr, bottom_left, ret[3]);
 
 #undef TRANSFORM_POINT
 
@@ -957,6 +1012,8 @@ graphene_matrix_transform_rect (const graphene_matrix_t *m,
  * The result is the axis aligned bounding rectangle containing the coplanar
  * quadrilateral.
  *
+ * See also: graphene_matrix_transform_point()
+ *
  * Since: 1.0
  */
 void
@@ -968,6 +1025,10 @@ graphene_matrix_transform_bounds (const graphene_matrix_t *m,
   float min_x, min_y;
   float max_x, max_y;
 
+  graphene_rect_t rr;
+
+  graphene_rect_normalize_r (r, &rr);
+
 #define TRANSFORM_POINT(matrix, rect, corner, out_p)   do {\
   graphene_simd4f_t __s; \
   graphene_point_t __p; \
@@ -977,10 +1038,10 @@ graphene_matrix_transform_bounds (const graphene_matrix_t *m,
   out_p.x = graphene_simd4f_get_x (__s); \
   out_p.y = graphene_simd4f_get_y (__s);           } while (0)
 
-  TRANSFORM_POINT (m, r, top_left, ret[0]);
-  TRANSFORM_POINT (m, r, top_right, ret[1]);
-  TRANSFORM_POINT (m, r, bottom_right, ret[2]);
-  TRANSFORM_POINT (m, r, bottom_left, ret[3]);
+  TRANSFORM_POINT (m, &rr, top_left, ret[0]);
+  TRANSFORM_POINT (m, &rr, top_right, ret[1]);
+  TRANSFORM_POINT (m, &rr, bottom_right, ret[2]);
+  TRANSFORM_POINT (m, &rr, bottom_left, ret[3]);
 
 #undef TRANSFORM_POINT
 
@@ -1145,7 +1206,7 @@ graphene_matrix_project_point (const graphene_matrix_t *m,
  * Projects a #graphene_rect_t using the given matrix.
  *
  * The resulting rectangle is the axis aligned bounding rectangle capable
- * of containing fully the projected rectangle.
+ * of fully containing the projected rectangle.
  *
  * Since: 1.0
  */
@@ -1156,11 +1217,14 @@ graphene_matrix_project_rect_bounds (const graphene_matrix_t *m,
 {
   graphene_point_t points[4];
   graphene_point_t ret[4];
+  graphene_rect_t rr;
 
-  graphene_rect_get_top_left (r, &points[0]);
-  graphene_rect_get_top_right (r, &points[1]);
-  graphene_rect_get_bottom_left (r, &points[2]);
-  graphene_rect_get_bottom_right (r, &points[3]);
+  graphene_rect_normalize_r (r, &rr);
+
+  graphene_rect_get_top_left (&rr, &points[0]);
+  graphene_rect_get_top_right (&rr, &points[1]);
+  graphene_rect_get_bottom_left (&rr, &points[2]);
+  graphene_rect_get_bottom_right (&rr, &points[3]);
 
   graphene_matrix_project_point (m, &points[0], &ret[0]);
   graphene_matrix_project_point (m, &points[1], &ret[1]);
@@ -1185,7 +1249,9 @@ graphene_matrix_project_rect_bounds (const graphene_matrix_t *m,
  * @res: (out caller-allocates): return location for the projected
  *   rectangle
  *
- * Projects a #graphene_rect_t using the given matrix.
+ * Projects all corners of a #graphene_rect_t using the given matrix.
+ *
+ * See also: graphene_matrix_project_point()
  *
  * Since: 1.2
  */
@@ -1195,17 +1261,20 @@ graphene_matrix_project_rect (const graphene_matrix_t *m,
                               graphene_quad_t         *res)
 {
   graphene_point_t p[4];
+  graphene_rect_t rr;
 
-  graphene_rect_get_top_left (r, &p[0]);
+  graphene_rect_normalize_r (r, &rr);
+
+  graphene_rect_get_top_left (&rr, &p[0]);
   graphene_matrix_project_point (m, &p[0], &p[0]);
 
-  graphene_rect_get_top_right (r, &p[1]);
+  graphene_rect_get_top_right (&rr, &p[1]);
   graphene_matrix_project_point (m, &p[1], &p[1]);
 
-  graphene_rect_get_bottom_left (r, &p[2]);
+  graphene_rect_get_bottom_left (&rr, &p[2]);
   graphene_matrix_project_point (m, &p[2], &p[2]);
 
-  graphene_rect_get_bottom_right (r, &p[3]);
+  graphene_rect_get_bottom_right (&rr, &p[3]);
   graphene_matrix_project_point (m, &p[3], &p[3]);
 
   graphene_quad_init_from_points (res, p);
