@@ -139,26 +139,20 @@ update_term_caps (void)
   global_state.is_tty = isatty (STDOUT_FILENO);
 #endif
 
+  global_state.use_colors = false;
+
   if (!global_state.is_tty)
-    {
-      global_state.use_colors = false;
-      return;
-    }
+    return;
 
   char *env = mutest_getenv ("MUTEST_NO_COLOR");
   if (env != NULL && env[0] != '\0')
-    {
-      global_state.use_colors = false;
-      free (env);
-      return;
-    }
+    goto out;
 
   env = mutest_getenv ("FORCE_COLOR");
   if (env != NULL && env[0] != '\0')
     {
       global_state.use_colors = true;
-      free (env);
-      return;
+      goto out;
     }
 
   env = mutest_getenv ("TERM");
@@ -174,19 +168,18 @@ update_term_caps (void)
           global_state.use_colors = true;
         }
 
-      free (env);
-      return;
+      goto out;
     }
 
   env = mutest_getenv ("COLORTERM");
   if (env != NULL)
     {
       global_state.use_colors = true;
-      free (env);
-      return;
+      goto out;
     }
 
-  global_state.use_colors = false;
+out:
+  free (env);
 }
 
 static void
@@ -200,10 +193,34 @@ update_term_size (void)
 
   if (ioctl (STDOUT_FILENO, TIOCGWINSZ, &ws) != 0)
     perror ("ioctl");
-
-  global_state.term_width = ws.ws_col;
-  global_state.term_height = ws.ws_row;
+  else
+    {
+      global_state.term_width = ws.ws_col;
+      global_state.term_height = ws.ws_row;
+      return;
+    }
 #endif
+
+  char *env = mutest_getenv ("COLUMNS");
+
+  if (env == NULL || *env == '\0')
+    {
+      free (env);
+      return;
+    }
+
+  int saved_errno = errno;
+  int columns = strtol (env, NULL, 10);
+  if (errno == ERANGE)
+    {
+      free (env);
+      return;
+    }
+
+  global_state.term_width = columns;
+  free (env);
+
+  errno = saved_errno;
 }
 
 static void
@@ -228,6 +245,7 @@ update_output_format (void)
     {
       // Default
       global_state.output_format = available_formats[0].format;
+      free (env);
       return;
     }
 
